---------------------------
-- lets simplify a bit..
---------------------------
local gf = Glance.Functions
local gv = Glance.Variables
local ga = Glance.Arrays
local gb = Glance.Buttons

---------------------------
-- create the button
---------------------------
gf.AddButton("Pets","RIGHT")
local btn = gb.Pets
btn.text              = "      "
btn.enabled           = true
btn.texture.normal    = "Interface\\AddOns\\Glance_Pets\\paw.tga"
btn.events            = {"COMPANION_LEARNED","PET_JOURNAL_AUTO_SLOTTED_PET","PET_JOURNAL_LIST_UPDATE","PET_JOURNAL_PET_DELETED"}
btn.update            = true
btn.click             = true
btn.tooltip           = true
btn.menu              = true
btn.onload            = true
btn.save.perAccount   = {["Pets"] = {}}

---------------------------
-- shortcuts
---------------------------
local spa = btn.save.perAccount
local HEX = ga.colors.HEX
local QA = ga.colors.QUALITY
local tooltip = gf.Tooltip

---------------------------
-- variables
---------------------------
gv.setShowPet = false
ga.pet = {}
gv.PetScrollIDX = 0
gv.PetPanelIDX = 0
gv.PetCount = 0
ga.PetQualityCount = {
	[1] = 0,
	[2] = 0,
	[3] = 0,
	[4] = 0,
	[5] = 0,
	[6] = 0,
}

---------------------------
-- update
---------------------------
function gf.Pets.update(self, event, arg1)
	if event == "COMPANION_LEARNED" then
		gf.Pets.resetPetList(event)
		gf.createMenu("Pets")
	end
end

---------------------------
-- tooltip
---------------------------
function gf.Pets.tooltip()
	Glance.Debug("function","tooltip","Pets")
	if gv.PetCount ~= C_PetJournal.GetNumPets(false) then
		gf.Pets.resetPetList()
		gf.createMenu("Pets")
	end
	tooltip.Title("Pets", "GLD")
	tooltip.Wrap("You have obtained "..HEX.green..gv.PetCount..HEX.white.." companion pets.","WHT")
	tooltip.Wrap(gf.Pets.getBreakdown(),"GLD")
	tooltip.Space()
	tooltip.Line("Legend", "GLD")
	tooltip.Line("Pets marked with an "..HEX.gold.."*|r are favorites.","WHT")
	tooltip.Line(HEX.red.."Red |rpets cannot be used as battle pets.","WHT")
	tooltip.Line("Pets are otherwise colored by their "..QA[1].."q"..QA[2].."u"..QA[3].."a"..QA[4].."l"..QA[5].."i"..QA[6].."t|ry.","WHT")
	tooltip.Notes("summon a random pet",nil,"summon a specific pet","hide pets","This list is controlled by the filters in your Pet Journal")
end

---------------------------
-- tooltip quality count
---------------------------
function gf.Pets.getBreakdown()
	local b = ""
	if ga.PetQualityCount[6] > 0 then b = b.."|r"..tostring(ga.PetQualityCount[6])..QA[6].." Legendary. " end
	if ga.PetQualityCount[5] > 0 then b = b.."|r"..tostring(ga.PetQualityCount[5])..QA[5].." Epic. " end
	if ga.PetQualityCount[4] > 0 then b = b.."|r"..tostring(ga.PetQualityCount[4])..QA[4].." Rare. " end
	if ga.PetQualityCount[3] > 0 then b = b.."|r"..tostring(ga.PetQualityCount[3])..QA[3].." Uncommon. " end
	if ga.PetQualityCount[2] > 0 then b = b.."|r"..tostring(ga.PetQualityCount[2])..QA[2].." Common. " end
	if ga.PetQualityCount[1] > 0 then b = b.."|r"..tostring(ga.PetQualityCount[1])..QA[1].." Poor. " end
	return b
end

---------------------------
-- click
---------------------------
function gf.Pets.click(self, button, down)
	Glance.Debug("function","click","Pets")
	if button == "LeftButton" then
		--TogglePetJournal(2) --SummonRandomCritter()
		C_PetJournal.SummonRandomPet(true);
	elseif button == "RightButton" then
		if IsShiftKeyDown() then
			gv.setShowPet = true
		else
			gv.setShowPet = false
		end
	end
end

---------------------------
-- onload
---------------------------
function gf.Pets.onload()
	gf.Pets.resetPetList()
end

---------------------------
-- menu
---------------------------
function gf.Pets.menu(level,UIDROPDOWNMENU_MENU_VALUE)
	Glance.Debug("function","menu","Pets")
	level = level or 1
	if (level == 1) then
		gf.setMenuTitle("Pets Options")
		gf.setMenuHeader("Favorites","Favorites",level)
		gf.setMenuHeader("Aquatic","Aquatic",level)
		gf.setMenuHeader("Beast","Beast",level)
		gf.setMenuHeader("Critter","Critter",level)
		gf.setMenuHeader("Dragonkin","Dragonkin",level)
		gf.setMenuHeader("Elemental","Elemental",level)
		gf.setMenuHeader("Flying","Flying",level)
		gf.setMenuHeader("Humanoid","Humanoid",level)
		gf.setMenuHeader("Magic","Magic",level)
		gf.setMenuHeader("Mechanical","Mechanical",level)
		gf.setMenuHeader("Undead","Undead",level)
		if not gv.setShowPet then gf.setInfo(nil,nil,nil,true,nil,HEX.red.."Dismiss Pet","dis",level,function() DoEmote("WAVE","pet"); C_PetJournal.SummonPetByGUID(C_PetJournal.GetSummonedPetGUID()) end) end --DismissCompanion("CRITTER")
	end
	if (level == 2) then
		for i=1,#ga.pet do
			local id, creatureName, icon, issummoned, enabled, favorite, lvl, petID, petType, canBattle, rarity = unpack(ga.pet[i])	
			local color = gf.getCondition(favorite, HEX.gold.."* "..QA[rarity], "   "..QA[rarity])
			if (not canBattle) then color = "   "..HEX.red	end			
			local ks = gf.getCondition(gv.setShowPet, true, false)
			local nr = gf.getCondition(gv.setShowPet, true, false)
			if gf.isMenuValue(_G["BATTLE_PET_NAME_"..petType]) and (enabled or gv.setShowPet) then
				if ks then
					gf.setMenuCheckBox(spa.Pets[creatureName],color..creatureName..HEX.gold.." ("..lvl..")",name,level,function() spa.Pets[creatureName] = gf.getCondition(spa.Pets[creatureName], false, true); gf.Pets.resetPetList(); end,icon,ks,nr)
				else
					gf.setMenuOption(C_PetJournal.GetSummonedPetGUID() == petID,color..creatureName..HEX.gold.." ("..lvl..")",name,level,function() gf.Pets.summon(id); end,icon)
				end
			end
			if gf.isMenuValue("Favorites") and enabled and favorite then	
				if ks then
					gf.setMenuCheckBox(spa.Pets[creatureName],color..creatureName..HEX.gold.." ("..lvl..")",name,level,function() spa.Pets[creatureName] = gf.getCondition(spa.Pets[creatureName], false, true); gf.Pets.resetPetList(); end,icon,ks,nr)
				else
					gf.setMenuOption(C_PetJournal.GetSummonedPetGUID() == petID,color..creatureName..HEX.gold.." ("..lvl..")",name,level,function() gf.Pets.summon(id); end,icon)
				end
			end
		end
	end
end

---------------------------
-- is summoned
---------------------------
function gf.Pets.isSummoned(id)
	if C_PetJournal.GetSummonedPetGUID() == id then
		return true
	else
		return false
	end
end

---------------------------
-- summon
---------------------------
function gf.Pets.summon(id)
	local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("CRITTER", id);
	if issummoned then
		--DismissCompanion("CRITTER")
		DoEmote("WAVE","pet"); 
		C_PetJournal.SummonPetByGUID(C_PetJournal.GetSummonedPetGUID())
	else
		local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle, tradable, unique = C_PetJournal.GetPetInfoByIndex(id, false)
		C_PetJournal.SummonPetByGUID(petID)
	end

end

---------------------------
-- get pet info
---------------------------
function gf.Pets.petInfo(id)
	local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle, tradable, unique = C_PetJournal.GetPetInfoByIndex(id, false)
	local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID);
	if C_PetJournal.GetSummonedPetGUID() == petID then
		issummoned = true
	else
		issummoned = false
	end
	local showPet = true
	if spa.Pets[name] ~= nil then
		showPet = spa.Pets[name]
	end
	return id, name, icon, issummoned, showPet, favorite, level, petID, petType, canBattle, rarity
end

---------------------------
-- store pets in {}
---------------------------
function gf.Pets.resetPetList(event)
	Glance.Debug("function","resetPetList",nil)
	wipe(ga.pet)
    C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_COLLECTED, true) -- displays pets we do have
    C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_NOT_COLLECTED, false) -- ignores pets we don't have
	gv.PetCount = C_PetJournal.GetNumPets(false)
	for i=1,6 do
		ga.PetQualityCount[i]=0
	end
	for i=1,gv.PetCount do
		ga.pet[i] = { gf.Pets.petInfo(i) }
		ga.PetQualityCount[ga.pet[i][11]] = ga.PetQualityCount[ga.pet[i][11]] + 1
		if spa.Pets[ga.pet[i][2]] == nil then
			spa.Pets[ga.pet[i][2]] = true
			if event == "COMPANION_LEARNED" then
				gf.sendMSG(ga.pet[i][2].." has been added to your pet list.")
			end
		end
	end
end

---------------------------
-- load on demand
---------------------------
if gv.loaded then
	gf.Enable("Pets")
end